// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:wyatt_component_copy_with_extension/wyatt_component_copy_with_extension.dart';
import 'package:wyatt_ui_components/wyatt_ui_components.dart';

part 'gradient_icon.g.dart';

@ComponentCopyWithExtension()
class GradientIcon extends GradientIconComponent with $GradientIconCWMixin {
  GradientIcon({
    required super.icon,
    required super.gradientColors,
    super.key,
    super.size,
    super.fill,
    super.weight,
    super.grade,
    super.opticalSize,
    super.color,
    super.shadows,
    super.semanticLabel,
    super.textDirection,
  }) : widgetIcon = null;

  GradientIcon.fromWidget(
    Widget icon, {
    MultiColor? gradientColors,
    super.key,
    super.size,
    super.fill,
    super.weight,
    super.grade,
    super.opticalSize,
    super.color,
    super.shadows,
    super.semanticLabel,
    super.textDirection,
  })  : widgetIcon = icon,
        super(
          icon: null,
          gradientColors: gradientColors ?? const MultiColor([]),
        );

  factory GradientIcon.fromIcon(
    Icon icon, {
    MultiColor? gradientColors,
  }) =>
      GradientIcon(
        icon: icon.icon,
        gradientColors: gradientColors ?? const MultiColor([]),
        key: icon.key,
        size: icon.size,
        fill: icon.fill,
        weight: icon.weight,
        grade: icon.grade,
        opticalSize: icon.opticalSize,
        color: icon.color,
        shadows: icon.shadows,
        semanticLabel: icon.semanticLabel,
        textDirection: icon.textDirection,
      );

  /// Creates a [GradientIcon] from a [Widget].
  ///
  /// Use [GradientIcon.fromWidget] to set this widget as the icon.
  final Widget? widgetIcon;

  @override
  Widget build(BuildContext context) {
    if (widgetIcon != null) {
      // Return the widget icon colorized with the super.color.
      if (super.color != null) {
        return ColorFiltered(
          colorFilter: ColorFilter.mode(
            super.color!,
            BlendMode.srcIn,
          ),
          child: widgetIcon,
        );
      }
      // else, return the widget icon with the gradient.
      final Gradient? linearGradient =
          GradientHelper.linearFromMultiColor(gradientColors);
      if (linearGradient != null) {
        return ShaderMask(
          blendMode: BlendMode.srcIn,
          shaderCallback: (bounds) => linearGradient.createShader(
            Rect.fromLTWH(0, 0, bounds.width, bounds.height),
          ),
          child: widgetIcon,
        );
      }

      if (gradientColors.colors.isNotEmpty) {
        // if multicolor has only one color, return the widget icon with the
        // color.
        return ColorFiltered(
          colorFilter: ColorFilter.mode(
            gradientColors.color,
            BlendMode.srcIn,
          ),
          child: widgetIcon,
        );
      }

      // If there is no gradient and no color, just build the widget icon.
      return widgetIcon!;
    }

    // if no widgetIcon, work with the icon.

    if (super.color != null) {
      return super.copyWith.call(color: super.color).build(context);
    }

    final Gradient? linearGradient =
        GradientHelper.linearFromMultiColor(gradientColors);
    if (linearGradient != null) {
      return ShaderMask(
        blendMode: BlendMode.srcIn,
        shaderCallback: (bounds) => linearGradient.createShader(
          Rect.fromLTWH(0, 0, bounds.width, bounds.height),
        ),
        child: super.build(context),
      );
    }

    if (gradientColors.colors.isNotEmpty) {
      return super.copyWith.call(color: gradientColors.color).build(context);
    }

    // If there is no gradient and no color, just build the icon.
    return super.build(context);
  }
}
